require 'httparty'

require 'appforce/config'
require 'appforce/logger'
require 'appforce/spawn/version'
require 'appforce/spawn/exceptions'
require 'appforce/spawn/api'
require 'appforce/spawn/template'
require 'appforce/spawn/runner'

module Appforce
  module Spawn

    ##
    # Extend module base class
    # Allows fo r class like interactions on Module
    def self.included(base)
      base.extend ClassMethods
    end

    # Define methods to extend
    module ClassMethods

      def logger
        Appforce::Logger.logger
      end

      def logger=(logger)
        Appforce::Logger.logger = logger
      end

      def header(input)
        Appforce::Logger.header(input)
      end

      def footer
        Appforce::Logger.footer
      end

      def ping_host
        HTTParty.get("#{config.api_host}/ping")
      end

      def ping_api
        Appforce::Spawn::Api::Call.ping
      end

      def load_config(*args)
        Appforce::Config.load_config(*args)
      end

      def config
        Appforce::Config.config
      end

      def clients
        Appforce::Spawn::Api::Call.list_clients
      end

      def get_clients
        Appforce::Spawn::Api::Call.get_clients
      end

      def get_hosts(*args)
        Appforce::Spawn::Api::Call.get_hosts(*args)
      end

      def get_active_users(*args)
        Appforce::Spawn::Api::Call.get_active_users(*args)
      end

      def get_inactive_users(*args)
        Appforce::Spawn::Api::Call.get_inactive_users(*args)
      end

      def get_vars(*args)
        Appforce::Spawn::Api::Call.get_vars(*args)
      end

      def generate(*args)
        logger.info "[#{self.name}##{__method__.to_s}] Template Generation Starting"
        Appforce::Spawn::Template.create_dirs(*args)
        Appforce::Spawn::Template.copy_template(*args)
        Appforce::Spawn::Template.download_files(*args)
        logger.info "[#{self.name}##{__method__.to_s}] Template Generation Complete"
      end

      def spawn(*args)
        Appforce::Spawn::Runner.can_run?
        Appforce::Spawn::Runner.run_playbook
      end

      def display_ansible_command(*args)
        Appforce::Spawn::Runner.can_run?
        Appforce::Spawn::Runner.display_ansible_command
      end

      def ansible_ping(*args)
        Appforce::Spawn::Runner.can_run?
        Appforce::Spawn::Runner.ansible_ping
      end

      def display_ansible_ping_command(*args)
        Appforce::Spawn::Runner.can_run?
        Appforce::Spawn::Runner.display_ansible_ping_command
      end

      def dump_example_config(*args)
        Appforce::Config.dump_example_config
      end

      def get_all_host_data(*args)
        Appforce::Spawn::Api::Call.get_all_host_data(*args)
      end

      def ssh_to_host(*args)
        Appforce::Spawn::Api::Call.ssh_to_host(*args)
      end

      def latest_version(*args)
        Appforce::Spawn::Api::Call.latest_version
      end
      
      def check_dependencies(*args)
        Appforce::Spawn::Runner.check_dependencies
        logger.info "[#{self.name}##{__method__.to_s}] All required Ansible checks passed."
      end
    end

    # Extension class
    class Basement #:nodoc:
      include Appforce::Spawn
    end

    def self.logger
      Basement.logger
    end

    def self.logger=(logger)
      Basement.logger = logger
    end

    def self.header(input)
      Basement.header(input)
    end

    def self.footer
      Basement.footer
    end

    def self.load_config(*args, &block)
      Basement.load_config(*args, &block)
    end

    def self.config(*args)
      Basement.config
    end

    def self.available_actions
      {
        'clients' => {
          info: "Retrieve a complete list of Clients",
          cmd:  :clients
        },
        'generate' => {
          info: "Build out the Ansible template for a Client (client API name is required)",
          cmd:  :generate
        },
        'client:hosts' => {
          info: "Retrieve a Hosts file for a Client (client API name is required)",
          cmd:  :get_hosts
        },
        'client:hosts:details' => {
          info: "Display all details about Client Hosts (client API name is required)",
          cmd:  :get_all_host_data
        },
        'client:users:active' => {
          info: "Retrieve an Active Users file for a Client (client API name is required)",
          cmd:  :get_active_users
        },
        'client:users:inactive' => {
          info: "Retrieve an Inactive Users file for a Client (client API name is required)",
          cmd:  :get_inactive_users
        },
        'client:vars' => {
          info: "Retrieve a Vars file for a Client (client API name is required)",
          cmd:  :get_vars
        },
        'ping' => {
          info: "Test connection to host and API access",
          cmd:  :ping
        },
        'ping:host' => {
          info: "Test if API is available",
          cmd:  :ping_host
        },
        'ping:api' => {
          info: "Test if you have access to the API",
          cmd:  :ping_api
        },
        'spawn' => {
          info: "Run Ansible Playbook to spawn users to Client hosts",
          cmd:  :spawn
        },
        'spawn:ready?' => {
          info: "Check to see if you have the required ansible version and roles installed",
          cmd:  :check_dependencies
        },
        'spawn:command' => {
          info: "Display Ansible command to run Playbook for a Client",
          cmd:  :display_ansible_command
        },
        'spawn:ping' => {
          info: "Ansible ping Client hosts in the 'hosts' file",
          cmd:  :ansible_ping
        },
        'spawn:ping:command' => {
          info: "Display ansible ping command",
          cmd:  :display_ansible_ping_command
        },
        'ssh' => {
          info: "Select a Host to ssh in as your User name (client API name is required)",
          cmd:  :ssh_to_host
        },
      }
    end

    def self.clients(*args)
      Basement.clients
    end

    def self.get_clients(*args)
      Basement.get_clients
    end

    def self.get_hosts(*args)
      Basement.get_hosts(*args)
    end

    def self.get_active_users(*args)
      Basement.get_active_users(*args)
    end

    def self.get_inactive_users(*args)
      Basement.get_inactive_users(*args)
    end
    
    def self.get_vars(*args)
      Basement.get_vars(*args)
    end

    def self.get_all_host_data(*args)
      Basement.get_all_host_data(*args)
    end

    def self.generate(*args)
      Basement.generate(*args)
    end

    def self.spawn(*args)
      Basement.spawn
    end

    def self.display_ansible_command(*args)
      Basement.display_ansible_command
    end

    def self.ansible_ping(*args)
      Basement.ansible_ping
    end

    def self.display_ansible_ping_command(*args)
      Basement.display_ansible_ping_command
    end

    def self.dump_example_config(*args)
      Basement.dump_example_config(*args)
    end

    def self.ssh_to_host(*args)
      Basement.ssh_to_host(*args)
    end

    def self.latest_version(*args)
      Basement.latest_version(*args)
    end
    
    def self.check_dependencies(*args)
      Basement.check_dependencies(*args)
    end

    def self.ping(*args)
      host = Basement.ping_host
      api  = Basement.ping_api
      {
        :host => { :code => host.code, :body => host.body },
        :api  => { :code => api.code, :body => api.body }
      }
    end

    def self.ping_host(*args)
      host = Basement.ping_host
      { :code => host.code, :body => host.body }
    end

    def self.ping_api(*args)
      api = Basement.ping_api
      { :code => api.code, :body => api.body }
    end

  end
end
